#module  CC_FAO_OUTPUT  "V1.0"

/*|++CCFAOOUT.C

 Facility: 
	Fermilab Accelerator Control System.
	General VAX-11 C Library Routine.

 Abstract:
	These routines combine the $FAO/$FAOL system services with the
	LIB$PUT_OUTPUT routine to produce and output (to SYS$OUTPUT) a
	formatted string.  These versions pass the control string by
	reference rather than by descriptor, thus being easier to use
	with VAX-11 C.

 Environment:
	User mode, linked from FERMILIB into a user's program.
|--*/
/*
 Modification History:
 Author: F. Nagy			Creation date: 02-Sep-83

 V1.0	02-Sep-83  FJN	Created for use in ACNAUX and ACNETBOOT.
 V1.1	14-Aug-84  FJN	Eliminated the LIB_ARGPTR call by using pointer tricks
*/

/* Include Files: */
#include  vaxtypes			/* VAX-11 C Type extensions */
#include  descrip			/* VAX-11 Descriptors */
#include  descrip2			/* Additional descriptor defs. */

/* Local definitions and types: */
typedef unsigned long status;		/* Condition code status type */

					/* Test status for failure and
					   if so, return to caller */
#define  FAIL_RETURN(status)  if ($VMS_FAILURE( status)) return status

#define  SIZE		255		/* Maximum text string size */


/*+0CC_FAO_OUTPUT

 Functional Description:
	This routine combines the $FAO system service and LIB$PUT_OUTPUT
	to format a string and write it to SYS$OUTPUT in one call.

 Input Parameters:
	ctrstr - pointer to control string
	p<n> - arguments with parameters for format directives

 Implicit Inputs:
	NONE

 Output Parameters:
	Returns - completion status code is returned

 Implicit Outputs:
	NONE

 Condition Codes:
	All errors return the failing status code.

 Side Effects:
	NONE
-*/
/*+ CC_FAO_OUTPUT
 Format and output to SYS$OUTPUT a string of text using the $FAO
 system service and a set of parameters specified in the calling
 sequence.

    status.wlc.v = CC_FAO_OUTPUT( ctrstr.rt.ra, [p1.rz.v,...])

 status   returned completion status code.

 ctrstr   the control string for the $FAO service passed by
          reference.  The control string is assumed to be
          terminated by a NUL byte (contains 0).

 p1...pn  Optional parameters for the formatting directives.
          Passed by value or reference as determined from the
          format directive.

 The resulting string is limited to 255 characters.
-*/

CC_FAO_OUTPUT( ctrstr, p1 )
char *ctrstr;				/* Control string */
ulong p1;				/* First parameter */
{
register ulong *plist = 0;		/* Pointer to list of parameters */
register ubyte nargs;			/* Gets number of arguments */

/*
 Get number of arguments followed by pointer to the list of $FAO 
 parameters (or left as 0 if none).
*/
nargs = *( (long *) &ctrstr - 1);	/* Gets number of arguments */
if (nargs > 1)
    plist = &p1;			/* Have parameters, get pointer */

/*
  Now call the FAOL variety of this routine!
*/
return CC_FAOL_OUTPUT( ctrstr, plist);

}

/*+0CC_FAOL_OUTPUT

 Functional Description:
	This routine combines the $FAOL system service and LIB$PUT_OUTPUT
	to format a string and write it to SYS$OUTPUT in one call.

 Input Parameters:
	ctrstr - pointer to control string
	pl - pointer to list of longword parameters for the format directives.

 Implicit Inputs:
	NONE

 Output Parameters:
	Returns - completion status code is returned

 Implicit Outputs:
	NONE

 Condition Codes:
	All errors return the failing status code.

 Side Effects:
	NONE
-*/
/*+ CC_FAOL_OUTPUT
 Format and output to SYS$OUTPUT a string of text using the $FAOL
 system service and a list of parameters.

    status.wlc.v = CC_FAOL_OUTPUT( ctrstr.rt.ra, pl.rz.ra )

 status   returned completion status code.

 ctrstr   the control string for the $FAO service passed by
          reference.  The control string is assumed to be
          terminated by a NUL byte (contains 0).

 pl       List of longword parameters used by the format directives
          as (or to access) data.  Passed by reference.

 The resulting string is limited to 255 characters.
-*/

CC_FAOL_OUTPUT( ctrstr, pl )
char *ctrstr;				/* Control string */
ulong *pl;				/* Parameter list pointer */
{
status SYS$FAOL();			/* Format text string */
status LIB$PUT_OUTPUT();		/* Output text to SYS$OUTPUT: */
$DESCRIPTOR0( d_ctrstr);		/* Prototype fixed string descriptor */
char buffer[SIZE];			/* Text buffer */
$DESCRIPTORM( d_buffer, buffer);	/* Descriptor for text buffer */
status sts;				/* Completion status holder */

/*
  Setup fixed control string descriptor for the $FAOL system service by
  using the pointer to the control string and scanning it until the
  terminating NUL is reached.
*/
d_ctrstr.dsc$a_pointer = ctrstr;
while (*ctrstr++ != '\0')
    d_ctrstr.dsc$w_length++;

/*
  Now format the parameter data into a text string using the control string.
*/
if (pl == NULL)
    sts = SYS$FAO( &d_ctrstr, &d_buffer.dsc$w_length, &d_buffer, 0);
else
    sts = SYS$FAOL( &d_ctrstr, &d_buffer.dsc$w_length, &d_buffer, pl);
FAIL_RETURN( sts);

/*
  Finally, output the text to SYS$OUTPUT.
*/
return LIB$PUT_OUTPUT( &d_buffer);

}
