#module  strpad  "V1.0"

/*|++STRPAD.C

 Facility: 
	Fermilab Accelerator Control System - VAX/VMS ACNET
	Fermilab VAX-11 C Extension Library.

 Abstract:
	VAX-11 C routine to pad (in place) a string with spaces to a
	specified total string length.

 Environment:
	FERMICLIB routine.
|--*/
/*
 Modification History:
 Author: F. Nagy

 V1.0	07-Jan-84  FJN	Created out of ENDTAPE.C for FERMICLIB
*/

/******************
 * Include Files: *
 ******************/
#include  vaxtypes			/* VAX-11 C Type extensions */


/*+ STRPAD
 Pad (in place) a string with blanks until there are n characters
 in the string.

    strpad( string, nwant )
    char *string;
    int nwant;
 -or-
  strpad( string.rt.ra, nwant.rl.v )

 string   pointer to the NUL-terminated string to be padded.
 nwant    desired length of padded string.

 Padding is done in place in the existing string; thus the string
 buffer must contain space for nwant+1 characters (the extra is for
 the NUL terminator).
-*/

strpad( string, nwant )
char *string;				/* String pointer */
int nwant;				/* Desired length of padded string */
{
int strlen();				/* Length of string to NUL */
int nspaces = nwant - strlen( string);	/* Number of padding blanks to add */
char *strpterm();			/* Returns pointer to the NUL */
char *end;				/* Pointer to string terminator */

end = strpterm( string);		/* Point to string terminator */

while (nspaces-- > 0)
    *end++ = ' ';			/* Add blanks at end */

*end = '\0';				/* Terminate string with NUL */

}
