	OPTIONS	/EXTEND_SOURCE
	INTEGER*4 FUNCTION LIB_CVT_LT_STR( clinks, string, vformat )
	IMPLICIT NONE
C
C+ LIB_CVT_LT_STR
C Converts time from a longword count of seconds since January 1, 1972
C to a character string in one of two formats:
C
C  status = LIB_CVT_LT_STR( clinks, string [,format] )
C
C status    completion status, returned by value.
C               SS$_NORMAL    successful return
C               SS$_IVTIME    invalid time value
C               LIB$_STRTRU   partial success, output string missing
C                             more than just seconds portion
C
C clinks    Longword time in seconds since January 1, 1972.  Passed
C           by reference, read only.
C
C string    Return time as text string.  Passed by descriptor, write
C           only.  The format is controlled by the optional format
C           specifier argument.
C
C format    Optional selector of format of time in text string.  Passed
C           by value.  The default value (0) returns a 14 character
C           string in the format "dd-mmm-yy hhmm".  If the low bit is
C           set (value 1), then standard VMS ASCII time format is used:
C           "dd-mmm-yyyy hh:mm:ss.cc".
C
C If the output string must be truncated, no error status is returned.
C-
	INTEGER*4 clinks		!Input time in seconds
	CHARACTER*(*) string		!Output string
	LOGICAL*4 vformat		!Optional output format
C
	INCLUDE 	'($SSDEF)/NOLIST'
	INCLUDE 	'($LIBDEF)/NOLIST'
C
	CHARACTER*2 asc_num		!Holder for 2 digits 
	CHARACTER*4 asc_year		!Holder for 4 digit year
	CHARACTER*(*) FULLDATE_DT	!Skeleton for DownTime-style date
	PARAMETER (FULLDATE_DT = 'DD-MMM-YY HHMM')
	CHARACTER*(*) FULLDATE_VMS	!Skeleton for VMS-style date
	PARAMETER (FULLDATE_VMS = 'DD-MMM-YYYY HH:MM:SS.00')
	CHARACTER*24 tos		!Temporary output string
	CHARACTER*3 MONTH(12)		!Month names
C
	DATA		MONTH /'JAN', 'FEB', 'MAR', 'APR', 'MAY',
	1		       'JUN', 'JUL', 'AUG', 'SEP', 'OCT',
	2		       'NOV', 'DEC'/
C
	INTEGER*4 status		!Completion status
	INTEGER*4 OTS$CVT_L_TI		!Longword integer to ASCII routine
	LOGICAL*4 LIB_TST_ARG_DFT	!Tests for default argument
	LOGICAL*4 dt_format		!FALSE=DT/TRUE=VMS string format
	STRUCTURE /DATE_TIME/
	    INTEGER*2 year
	    INTEGER*2 month		!Month number
	    INTEGER*2 day		!Day of month
	    INTEGER*2 hour
	    INTEGER*2 minute
	    INTEGER*2 second
	END STRUCTURE
	RECORD /DATE_TIME/ cst		!Central Standard Time
	INTEGER*2 year			!Year
	INTEGER*2 nyear, ndays, i	!Intermediaries
	INTEGER*4 nsec			!Clinks for calculating with
	INTEGER*4 NSPM			!# of seconds/minute
	PARAMETER (NSPM = 60)
	INTEGER*4 NSPH			!# of seconds/hour
	PARAMETER (NSPH = NSPM * 60)
	INTEGER*4 NSPD			!# of seconds/day
	PARAMETER (NSPD = NSPH * 24)
	INTEGER*4 NSP4Y			!# of seconds in 4 years
	PARAMETER (NSP4Y = 4 * 365 * NSPD + NSPD)
	INTEGER*4 NSPY			!# of seconds in a short year
	PARAMETER (NSPY = 365 * NSPD)
	INTEGER*2 YRBIAS		!Zero time (Jan 1, 1972, MUST be leap year)
	PARAMETER (YRBIAS = 72)
	INTEGER*2 DAYSUM(13)		!Sum of days in year to start of each month
C		     j   f   m   a    m    j    j    a    s    o    n    d    j
	DATA DAYSUM/ 0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366/ 
C		  + 31  ??  31  30   31   30   31   31   30   31   30   31      !deltas  (good old February)
C
	string = ' '				!error return; will come out as Nov 17, 1858
	LIB_CVT_LT_STR = SS$_IVTIME		!Error status, preloaded
	IF( clinks .lt. 0 ) RETURN		!return, with error status
C
	IF (LIB_TST_ARG_DFT(3)) THEN
	    dt_format = %LOC(vformat) .EQ. 0	!If non-zero value then VMS format
	ELSE
	    dt_format = .TRUE.
	ENDIF

	nsec = clinks				!so as not to trash caller's value

	nyear = nsec / NSP4Y			!# of four-year cycles to nyear
	nsec = MOD( nsec, NSP4Y )		!adjust seconds to remainder
	year = YRBIAS + (4 * nyear)		!add quad years to bias (any remaining years are short)

	IF( nsec .lt. (NSPY+NSPD) ) THEN	!LEAP YEAR?
	    nyear = 0				!so don't add short year for it (nsec already < year)
	ELSE					!NOT A LEAP YEAR
	    nyear = (nsec-NSPD) / NSPY		!get count of remaining short years
	    nsec = nsec - (nyear * NSPY) - NSPD	!adjust seconds to remainder (now < a year)
	ENDIF
	year = year + nyear			!add short years to count
C===	IF( year .ge. 100 ) year = year - 100	!maintain two-digit year coding - NO DON'T
	cst.year = year				!and put into CST record

	ndays = nsec / NSPD			!number of whole days to 'ndays'
	nsec = nsec - (ndays * NSPD)		!adjust seconds to remainder (now less than a day)
	IF( MOD(year,4) .ne. 0 ) THEN		!not a leap year?
	    IF( ndays .ge. DAYSUM(3)-1 ) THEN	!Mar-Dec ?
		ndays = ndays + 1
	    ENDIF
	ENDIF
	DO i = 12, 1, -1			!scan through table of sums of days by month
	    IF( DAYSUM(i) .le. ndays ) THEN	!Found the month?
		cst.month = i			!stuff into CST
		cst.day = ndays - DAYSUM(i) + 1	!day in month (first day in month is one, not zero)
		cst.hour = nsec / NSPH		!hour of day 
		nsec = nsec - cst.hour*NSPH	!adjust nsec to remainder
		cst.minute = nsec / NSPM	!minute of hour 
		nsec = nsec - cst.minute*NSPM	!adjust nsec to remainder
		cst.second = nsec		!second of minute 
		GOTO 99				!all done. Got integer values for date.
	  ENDIF
	ENDDO
C
C Should NEVER get here - but return w/ error status
C
	RETURN
C
99	CONTINUE
C
C Load output string with skeleton format
C
	IF (dt_format) THEN
	    tos = FULLDATE_DT
	ELSE
	    tos = FULLDATE_VMS
	ENDIF
C
C Convert year from integer to ASCII.
C
	IF (.NOT. dt_format) cst.year = cst.year + 1900
	status = OTS$CVT_L_TI ( cst.year, asc_year, %VAL(4), %VAL(2))
	IF (.NOT. status) RETURN
	IF (dt_format) THEN
	    tos(8:9) = asc_year(3:4)	!poke the year into DT date phrase
	ELSE
	    tos(8:11) = asc_year	!poke complete year into VMS date phrase
	ENDIF
C
	tos(4:6) = MONTH(cst.month)	!poke month into phrase

	status = OTS$CVT_L_TI( cst.day, asc_num, %VAL(2), %VAL(2))
	IF (.NOT. status) RETURN
	tos(1:2) = asc_num		!day

	status = OTS$CVT_L_TI( cst.hour, asc_num, %VAL(2), %VAL(2))
	IF (.NOT. status) RETURN
	IF (dt_format) THEN
	    tos(11:12) = asc_num	!hour - DT
	ELSE
	    tos(13:14) = asc_num	!hour - VMS
	ENDIF

	status = OTS$CVT_L_TI( cst.minute, asc_num, %VAL(2), %VAL(2))
	IF (.NOT. status) RETURN
	IF (dt_format) THEN
	    tos(13:14) = asc_num	!minute - DT
	ELSE
	    tos(16:17) = asc_num	!minute - VMS
	ENDIF
C
C Everything looks okay. Store the output string, checking for truncation.
C
	LIB_CVT_LT_STR = SS$_NORMAL	!Assumes success for now
	string = tos
	IF (dt_format) THEN
	    i = LEN(FULLDATE_DT) - 2	!"SS" field may be left off
	ELSE
	    i = LEN(FULLDATE_VMS) - 6	!":SS.00" may be left off
	ENDIF
	IF (LEN(string) .LT. i) THEN
	    LIB_CVT_LT_STR = LIB$_STRTRU    !String was truncated!
	    RETURN
	ENDIF
C
C Check to see if seconds are to be returned
C
	IF (LEN(string) .GT. i) THEN
	    status = OTS$CVT_L_TI( cst.second, asc_num, %VAL(2), %VAL(2))
	    IF (.NOT. status) THEN
		LIB_CVT_LT_STR = SS$_IVTIME
		RETURN
	    ENDIF
	    IF (dt_format) THEN
		string(15:16) = asc_num	!seconds - DT
	    ELSE
		string(19:20) = asc_num	!seconds - VMS
	    ENDIF
	ENDIF
C
	RETURN
	END
C
C Author:	Al Thomas	Fermilab Accelerator/Operations
C Modification History:
C V1.0	22-Mar-86  FJN	Renamed as LIB_CVT_LT_STR, calling sequence modified,
C			and setup to be added to FERMILIB and installed as a
C			Datatrieve function.
