#module  LIB_CREATE_LOGICAL_NAME  "V1.0"

/* LIBCRELNM.C

 Facility: 
	Fermilab Accelerator Control System - VAX/VMS ACNET
	General library routine

 Abstract:
	This routine provides a simple interface to $CRELNM system service.
	It can be used to replace LIB$SET_LOGICAL in detached processes with
	no CLI.

 Environment:
	Called from user mode usually.
*/
/*
 Author: Frank J. Nagy		Fermilab Accelerator Controls
 Modification History:

 V1.0	01-Aug-86  FJN	Created for AEOLUS and DBM where LIB$SET_LOGICAL will
			not work
*/

/******************
 * Include Files: *
 ******************/
#include  vaxtypes			/* VAX-11 C Type extensions */
#include  descrip			/* VAX-11 Descriptors */
#include  descrip2			/* Additional descriptor defs. */
#include  lnmdef			/* Logical Name definitions */

/********************************
 * Local definitions and types: *
 ********************************/
typedef unsigned long status;		/* Condition code status type */

struct an_item {			/* Element of an item list */
    uword buflen;			/* Item buffer length */
    uword itmcod;			/* Item code */
    address bufferp;			/* Pointer to item's buffer */
    ulong *rtnlenp;			/* Pointer to return length */
    };

typedef struct dsc$descriptor *dscptr;	/* Pointer to descriptor */

/*****************************
 * Module Local Own Storage: *
 *****************************/

readonly static $DESCRIPTOR( d_lnm$process, "LNM$PROCESS");

/**********************************************
 * Global Procedure and Function definitions: *
 **********************************************/
status SYS$CRELNM();			/* Create logical name service */

status LIB$ANALYZE_SDESC();		/* Analyze string descriptor */


/*+ LIB_CRELNM
 Provide a simple (no item list) interface to $CRELNM to create a logical
 name with a simple equivalence string (a single item).  

   status.wlc.v = LIB_CRELNM( lognam.rt.dx, eqvnam.rt.dx
                              [,tattrs.rlu.r] [,nattrs.rlu.r]
                              [,tabnam.rt.dx] )

  status    completion status.  Returned by value.

  lognam    logical name to be created.  Passed by descriptor.

  eqvnam    simple name equivalence string for the logical name.
            Passed by descriptor.

  tattrs    mask of translation atrributes (such as LNM$M_CONCEALED,
            LNM$M_TERMINAL, etc.).  Passed by reference.
            Default is no translation attributes.

  nattrs    mask of atrributes to be associated with the logical name
            (such as LNM$M_CONFINE, LNM$M_NO_ALIAS, etc.).  Passed by
            reference.  Default is no attributes.

  tabnam    optional table name in which to create the logical name.
            Passed by descriptor.  Default is "LNM$PROCESS".

 More information can be found in the documentation on the $CRELNM system
 service.
-*/

#define	NARG_TATTRS	3		/* # of optional argument */
#define	NARG_NATTRS	4
#define	NARG_TABNAM	5

status LIB_CRELNM( lognamp, eqvnamp, tattrp, nattrp, tabnamp )
dscptr lognamp;				/* Descriptor for logical name */
dscptr eqvnamp;				/* Descriptor for equivalence name */
unsigned long *tattrp;			/* Pointer to equivalence attributes */
unsigned long *nattrp;			/* Pointer to name attributes */
dscptr tabnamp;				/* Descriptor for table name */
{
int nargs = *((long *) &lognamp - 1);	/* Number of arguments in call */
register status sts;
unsigned long *nap = NULL;		/* Logical name attributes pointer */
struct {
    struct an_item attributes;		/* Translation attributes */
    struct an_item estring;		/* Equivalence string item */
    unsigned long list_end;			/* End of item list */
    } item_list;
$DESCRIPTOR0( d_lognam);		/* Prototype logical name descriptor */
$DESCRIPTOR0( d_table );		/* Prototype table name descriptor */

/*
 Initialize the item list.
*/
item_list.attributes.buflen = 4;
item_list.attributes.bufferp = item_list.estring.bufferp = 0;
item_list.attributes.rtnlenp = item_list.estring.rtnlenp = 0;
item_list.attributes.itmcod = LNM$_ATTRIBUTES;
item_list.estring.itmcod = LNM$_STRING;
item_list.list_end = 0;

/*
 Check for optional table name or use default of LNM$PROCESS
*/
if ((nargs >= NARG_TABNAM) && (tabnamp != NULL))
    {
    sts = LIB$ANALYZE_SDESC( tabnamp, &d_table.dsc$w_length,
					&d_table.dsc$a_pointer);
    RETURN_ON_FAILURE( sts);
    }
else
    d_table = d_lnm$process;

/*
 Check for optional logical name attributes.
*/
if ((nargs >= NARG_NATTRS) && (nattrp!= NULL))
    nap = nattrp;			/* Use caller's name attributes */

/*
 Check for optional translation attributes.
*/
if ((nargs >= NARG_TATTRS) && (tattrp!= NULL))
    item_list.attributes.bufferp = (address) tattrp;
else
    item_list.attributes.bufferp = (address) &item_list.attributes.rtnlenp;

/*
 Get length and pointer to equivalence string.
*/
sts = LIB$ANALYZE_SDESC( eqvnamp, &item_list.estring.buflen,
				&item_list.estring.bufferp);
RETURN_ON_FAILURE( sts);

/*
 Fixup descriptor (fixed-length) for logical name.
*/
sts = LIB$ANALYZE_SDESC( lognamp, &d_lognam.dsc$w_length,
				&d_lognam.dsc$a_pointer);
RETURN_ON_FAILURE( sts);

return SYS$CRELNM( nap, &d_table, &d_lognam, 0, &item_list);
}
