#module  LIB_FAO_OUTPUT  "V1.1"

/*|++LIBFAOOUT.C

 Facility: 
	Fermilab Accelerator Control System.
	General Library Routine.

 Abstract:
	These routines combine the $FAO/$FAOL system services with the
	LIB$PUT_OUTPUT routine to produce and output (to SYS$OUTPUT) a
	formatted string.

 Environment:
	User mode, linked from FERMILIB into a user's program.
|--*/
/*
 Modification History:
 Author: F. Nagy			Creation date: 02-Sep-83

 V1.0	02-Sep-83  FJN	Created for use in ACNAUX and ACNETBOOT.
 V1.1	14-Aug-84  FJN	Eliminated LIB_ARGPTR call using pointer trick
*/

/* Include Files: */
#include  vaxtypes			/* VAX-11 C Type extensions */
#include  descrip			/* VAX-11 Descriptors */
#include  descrip2			/* Additional descriptor defs. */

/* Local definitions and types: */
typedef unsigned long status;		/* Condition code status type */

					/* Test status for failure and
					   if so, return to caller */
#define  FAIL_RETURN(status)  if ($VMS_FAILURE( status)) return status

#define  SIZE		255		/* Maximum text string size */


/*+0LIB_FAO_OUTPUT

 Functional Description:
	This routine combines the $FAO system service and LIB$PUT_OUTPUT
	to format a string and write it to SYS$OUTPUT in one call.

 Input Parameters:
	ctrstr - descriptor for control string
	p<n> - arguments with parameters for format directives

 Implicit Inputs:
	NONE

 Output Parameters:
	Returns - completion status code is returned

 Implicit Outputs:
	NONE

 Condition Codes:
	All errors return the failing status code.

 Side Effects:
	NONE
-*/
/*+ LIB_FAO_OUTPUT
 Format and output to SYS$OUTPUT a string of text using the $FAO
 system service and a set of parameters specified in the calling
 sequence.

    status.wlc.v = LIB_FAO_OUTPUT( ctrstr.rt.dx, [p1.rz.v,...])

 status   returned completion status code.

 ctrstr   the control string for the $FAO service passed by
          descriptor.  Any string descriptor class may be used.

 p1...pn  Optional parameters for the formatting directives.
          Passed by value or reference as determined from the
          format directive.

 The resulting string is limited to 255 characters.
-*/

LIB_FAO_OUTPUT( ctrstr, p1 )
struct dsc$descriptor *ctrstr;		/* Control string descriptor */
ulong p1;				/* First parameter */
{
register ulong *plist = 0;		/* Pointer to list of parameters */
register ubyte nargs;			/* Gets number of arguments */

/*
 Get number of arguments followed by pointer to the list of $FAO 
 parameters (or left as 0 if none).
*/
nargs = *( (long *) &ctrstr - 1);	/* Gets number of arguments */
if (nargs > 1)
    plist = &p1;			/* Have parameters, get pointer */

/*
 Now call the FAOL variety of this routine!
*/
return LIB_FAOL_OUTPUT( ctrstr, plist);

}

/*+0LIB_FAOL_OUTPUT

 Functional Description:
	This routine combines the $FAOL system service and LIB$PUT_OUTPUT
	to format a string and write it to SYS$OUTPUT in one call.

 Input Parameters:
	ctrstr - descriptor for control string
	pl - pointer to list of longword parameters for the format directives.

 Implicit Inputs:
	NONE

 Output Parameters:
	Returns - completion status code is returned

 Implicit Outputs:
	NONE

 Condition Codes:
	All errors return the failing status code.

 Side Effects:
	NONE
-*/
/*+ LIB_FAOL_OUTPUT
 Format and output to SYS$OUTPUT a string of text using the $FAOL
 system service and a list of parameters.

    status.wlc.v = LIB_FAOL_OUTPUT( ctrstr.rt.dx, pl.rz.ra )

 status   returned completion status code.

 ctrstr   the control string for the $FAO service passed by
          descriptor.  Any string descriptor class may be used.

 pl       List of longword parameters used by the format directives
          as (or to access) data.  Passed by reference.

 The resulting string is limited to 255 characters.
-*/

LIB_FAOL_OUTPUT( ctrstr, pl )
struct dsc$descriptor *ctrstr;		/* Control string descriptor */
ulong *pl;				/* Parameter list pointer */
{
status LIB$ANALYZE_SDESC();		/* Break up string descriptor */
status SYS$FAOL();			/* Format text string */
status LIB$PUT_OUTPUT();		/* Output text to SYS$OUTPUT: */
$DESCRIPTOR0( d_ctrstr);		/* Prototype fixed string descriptor */
char buffer[SIZE];			/* Text buffer */
$DESCRIPTORM( d_buffer, buffer);	/* Descriptor for text buffer */
status sts;				/* Completion status holder */

/*
  Analyze string descriptor of the control string and use it to make a
  fixed string descriptor for the $FAOL system service.
*/
sts = LIB$ANALYZE_SDESC( ctrstr, &d_ctrstr.dsc$w_length,
				 &d_ctrstr.dsc$a_pointer);
FAIL_RETURN( sts);

/*
  Now format the parameter data into a text string using the control string.
*/
if (pl == NULL)
    sts = SYS$FAO( &d_ctrstr, &d_buffer.dsc$w_length, &d_buffer, 0);
else
    sts = SYS$FAOL( &d_ctrstr, &d_buffer.dsc$w_length, &d_buffer, pl);
FAIL_RETURN( sts);

/*
  Finally, output the text to SYS$OUTPUT.
*/
return LIB$PUT_OUTPUT( &d_buffer);

}
